@@ -500,7 +500,7 @@ public class Collections {
 //        }
 //    }
 
-    private static class CheckedMap implements Map {
+    private static class CheckedMap implements Map, Serializable {
         final Map map;
         final Class keyType, valueType;
         transient Set entrySet;
@@ -774,7 +774,7 @@ public class Collections {
         public boolean remove(Object o)     { return deque.remove(o); }
         public Iterator iterator()          { return deque.iterator(); }
         public String toString()            { return deque.toString(); }
-	public boolean containsAll(Collection c) { return deque.containsAll(c); }
+	    public boolean containsAll(Collection c) { return deque.containsAll(c); }
         public boolean removeAll(Collection c)   { return deque.removeAll(c); }
         public boolean retainAll(Collection c)   { return deque.retainAll(c); }
     }
@@ -787,5 +787,13 @@ public class Collections {
         public int compare(Object o1, Object o2) {
             return cmp.compare(o2, o1);
         }
+        public boolean equals(Object o) {
+            return (o == this) ||
+            (o instanceof ReverseComparator &&
+             cmp.equals(((ReverseComparator)o).cmp));
+        }
+        public int hashCode() {
+            return cmp.hashCode() ^ 0x10000000;
+        }
     }
 }
