@@ -53,7 +53,7 @@ import java.util.ConcurrentModificationException;
  * @since 1.5
  * @author Dawid Kurzyniec
  */
-public class PriorityQueue extends AbstractQueue implements java.io.Serializable, Queue {
+public class PriorityQueue extends AbstractQueue implements java.io.Serializable {
 
     private final static long serialVersionUID = -7720805057305804111L;
 
@@ -296,7 +296,7 @@ public class PriorityQueue extends AbstractQueue implements java.io.Serializable
                     int c = (idx<<1)+1;
                     if (c >= size) break;
                     if (c+1 < size) {
-                        if (comparator.compare(buffer[c+1], buffer[c]) < 0) c++;
+                        if (comparator.compare(buffer[c], buffer[c+1]) > 0) c++;
                     }
                     if (comparator.compare(e, buffer[c]) <= 0) break;
                     buffer[idx] = buffer[c];
@@ -309,7 +309,7 @@ public class PriorityQueue extends AbstractQueue implements java.io.Serializable
                     int c = (idx<<1)+1;
                     if (c >= size) break;
                     if (c+1 < size) {
-                        if (((Comparable)buffer[c+1]).compareTo(buffer[c]) < 0) c++;
+                        if (((Comparable)buffer[c]).compareTo(buffer[c+1]) > 0) c++;
                     }
                     if (ec.compareTo(buffer[c]) <= 0) break;
                     buffer[idx] = buffer[c];
