@@ -41,7 +41,7 @@ import edu.emory.mathcs.backport.java.util.concurrent.helpers.*;
  * Iterator} interfaces.
  *
  * <p>This class is a member of the
- * <a href="{@docRoot}/../guide/collections/index.html">
+ * <a href="{@docRoot}/../technotes/guides/collections/index.html">
  * Java Collections Framework</a>.
  *
  * @since 1.5
@@ -61,9 +61,9 @@ public class ArrayBlockingQueue extends AbstractQueue
     /** The queued items  */
     private final Object[] items;
     /** items index for next take, poll or remove */
-    private transient int takeIndex;
+    private int takeIndex;
     /** items index for next put, offer, or add. */
-    private transient int putIndex;
+    private int putIndex;
     /** Number of items in the queue */
     private int count;
 
