@@ -1,4 +1,4 @@
-/*
+ /*
  * Written by Doug Lea with assistance from members of JCP JSR-166
  * Expert Group and released to the public domain, as explained at
  * http://creativecommons.org/licenses/publicdomain
@@ -17,7 +17,6 @@ import java.util.Map;
 import java.util.Enumeration;
 import java.util.Iterator;
 import java.util.NoSuchElementException;
-import edu.emory.mathcs.backport.java.util.concurrent.helpers.Utils;
 
 /**
  * A hash table supporting full concurrency of retrievals and
@@ -69,7 +68,7 @@ import edu.emory.mathcs.backport.java.util.concurrent.helpers.Utils;
  * does <em>not</em> allow <tt>null</tt> to be used as a key or value.
  *
  * <p>This class is a member of the
- * <a href="{@docRoot}/../guide/collections/index.html">
+ * <a href="{@docRoot}/../technotes/guides/collections/index.html">
  * Java Collections Framework</a>.
  *
  * @since 1.5
@@ -151,18 +150,21 @@ public class ConcurrentHashMap extends AbstractMap
     /* ---------------- Small Utilities -------------- */
 
     /**
-     * Returns a hash code for non-null Object x.
-     * Uses the same hash code spreader as most other java.util hash tables.
-     * @param x the object serving as a key
-     * @return the hash code
+     * Applies a supplemental hash function to a given hashCode, which
+     * defends against poor quality hash functions.  This is critical
+     * because ConcurrentHashMap uses power-of-two length hash tables,
+     * that otherwise encounter collisions for hashCodes that do not
+     * differ in lower or upper bits.
      */
-    static int hash(Object x) {
-        int h = x.hashCode();
-        h += ~(h << 9);
-        h ^=  (h >>> 14);
-        h +=  (h << 4);
-        h ^=  (h >>> 10);
-        return h;
+    private static int hash(int h) {
+        // Spread bits to regularize both segment and index locations,
+        // using variant of single-word Wang/Jenkins hash.
+        h += (h <<  15) ^ 0xffffcd7d;
+        h ^= (h >>> 10);
+        h += (h <<   3);
+        h ^= (h >>>  6);
+        h += (h <<   2) + (h << 14);
+        return h ^ (h >>> 16);
     }
 
     /**
@@ -764,7 +766,7 @@ public class ConcurrentHashMap extends AbstractMap
      * @throws NullPointerException if the specified key is null
      */
     public Object get(Object key) {
-        int hash = hash(key); // throws NullPointerException if key null
+        int hash = hash(key.hashCode()); // throws NullPointerException if key null
         return segmentFor(hash).get(key, hash);
     }
 
@@ -778,7 +780,7 @@ public class ConcurrentHashMap extends AbstractMap
      * @throws NullPointerException if the specified key is null
      */
     public boolean containsKey(Object key) {
-        int hash = hash(key); // throws NullPointerException if key null
+        int hash = hash(key.hashCode()); // throws NullPointerException if key null
         return segmentFor(hash).containsKey(key, hash);
     }
 
@@ -878,7 +880,7 @@ public class ConcurrentHashMap extends AbstractMap
     public Object put(Object key, Object value) {
         if (value == null)
             throw new NullPointerException();
-        int hash = hash(key); // throws NullPointerException if key null
+        int hash = hash(key.hashCode()); // throws NullPointerException if key null
         return segmentFor(hash).put(key, hash, value, false);
     }
 
@@ -892,7 +894,7 @@ public class ConcurrentHashMap extends AbstractMap
     public Object putIfAbsent(Object key, Object value) {
         if (value == null)
             throw new NullPointerException();
-        int hash = hash(key); // throws NullPointerException if key null
+        int hash = hash(key.hashCode()); // throws NullPointerException if key null
         return segmentFor(hash).put(key, hash, value, true);
     }
 
@@ -920,7 +922,7 @@ public class ConcurrentHashMap extends AbstractMap
      * @throws NullPointerException if the specified key is null
      */
     public Object remove(Object key) {
-        int hash = hash(key); // throws NullPointerException if key null
+        int hash = hash(key.hashCode()); // throws NullPointerException if key null
         return segmentFor(hash).remove(key, hash, null);
     }
 
@@ -932,7 +934,7 @@ public class ConcurrentHashMap extends AbstractMap
     public boolean remove(Object key, Object value) {
         if (value == null)
             return false;
-        int hash = hash(key); // throws NullPointerException if key null
+        int hash = hash(key.hashCode()); // throws NullPointerException if key null
         return segmentFor(hash).remove(key, hash, value) != null;
     }
 
@@ -944,7 +946,7 @@ public class ConcurrentHashMap extends AbstractMap
     public boolean replace(Object key, Object oldValue, Object newValue) {
         if (oldValue == null || newValue == null)
             throw new NullPointerException();
-        int hash = hash(key); // throws NullPointerException if key null
+        int hash = hash(key.hashCode()); // throws NullPointerException if key null
         return segmentFor(hash).replace(key, hash, oldValue, newValue);
     }
 
@@ -958,7 +960,7 @@ public class ConcurrentHashMap extends AbstractMap
     public Object replace(Object key, Object value) {
         if (value == null)
             throw new NullPointerException();
-        int hash = hash(key); // throws NullPointerException if key null
+        int hash = hash(key.hashCode()); // throws NullPointerException if key null
         return segmentFor(hash).replace(key, hash, value);
     }
 
@@ -1037,7 +1039,7 @@ public class ConcurrentHashMap extends AbstractMap
      * Returns an enumeration of the keys in this table.
      *
      * @return an enumeration of the keys in this table
-     * @see #keySet
+     * @see #keySet()
      */
     public Enumeration keys() {
         return new KeyIterator();
@@ -1047,7 +1049,7 @@ public class ConcurrentHashMap extends AbstractMap
      * Returns an enumeration of the values in this table.
      *
      * @return an enumeration of the values in this table
-     * @see #values
+     * @see #values()
      */
     public Enumeration elements() {
         return new ValueIterator();
