@@ -443,14 +443,18 @@ public class CopyOnWriteArrayList implements List, RandomAccess, Cloneable, Seri
         public int nextIndex() { return cursor; }
         public Object next() {
             try { return array[cursor++]; }
-            catch (IndexOutOfBoundsException e) { throw new NoSuchElementException(); }
-            // todo: should decrement cursor on failure?...
+            catch (IndexOutOfBoundsException e) {
+                --cursor;
+                throw new NoSuchElementException();
+            }
         }
         public int previousIndex() { return cursor-1; }
         public Object previous() {
             try { return array[--cursor]; }
-            catch (IndexOutOfBoundsException e) { throw new NoSuchElementException(); }
-            // todo: should decrement cursor on failure?...
+            catch (IndexOutOfBoundsException e) {
+                cursor++;
+                throw new NoSuchElementException();
+            }
         }
         public void add(Object val) {
             throw new UnsupportedOperationException();
